#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of the disease prevalence corresponding to the specified hypothesis. 
#The disease prevalence Kp (or probD) can be estimated from equation (3) in Gauderman, Statistics in Medicine, 2002, 21:35-50. 
Kp_est<-function(P0,or_T,or_E,int_TE,fT,fE,mode_inh,hypothesis){
    
    probD<-0
    alphaP0<-log(P0/(1-P0))
    beta1<-log(or_T)
    beta2<-log(or_E)
    beta3<-log(int_TE)
    
    if(hypothesis=="GE"|hypothesis=="GGE"){
        #Loop over all genotype-exposure combinations (2*3=6). 
        for(comb in 1:6){
            if(comb==1){
                gf<-"AA"
                freqT<-fT^2
                if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    G<-2
                }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                    G<-1
                }
                E<-1
                freqE<-fE
            }else if(comb==2){
                gf<-"aA"
                freqT<-2*fT*(1-fT)
                if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    G<-1
                }else if(mode_inh=="recessive"){
                    G<-0
                }
                E<-1
                freqE<-fE
            }else if(comb==3){
                gf<-"aa"
                freqT<-(1-fT)^2
                G<-0
                E<-1
                freqE<-fE
            }else if(comb==4){
                gf<-"AA"
                freqT<-fT^2
                if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    G<-2
                }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                    G<-1
                }
                E<-0
                freqE<-1-fE
            }else if(comb==5){
                gf<-"aA"
                freqT<-2*fT*(1-fT)
                if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    G<-1
                }else if(mode_inh=="recessive"){
                    G<-0
                }
                E<-0
                freqE<-1-fE
            }else if(comb==6){
                gf<-"aa"
                freqT<-(1-fT)^2
                G<-0
                E<-0
                freqE<-1-fE
            }

            probD<-probD+exp(alphaP0+beta1*G+beta2*E+beta3*G*E)/(1+exp(alphaP0+beta1*G+beta2*E+beta3*G*E))*freqT*freqE
        }    
    
    }else if(hypothesis=="G"){
        #Loop over all genotypes (3 possibilities). 
        for(comb in 1:3){
            if(comb==1){
                gf<-"AA"
                freqT<-fT^2
                if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    G<-2
                }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                    G<-1
                }
            }else if(comb==2){
                gf<-"aA"
                freqT<-2*fT*(1-fT)
                if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    G<-1
                }else if(mode_inh=="recessive"){
                    G<-0
                }
            }else if(comb==3){
                gf<-"aa"
                freqT<-(1-fT)^2
                G<-0
            }
            probD<-probD+exp(alphaP0+beta1*G)/(1+exp(alphaP0+beta1*G))*freqT
        }    
    }else if(hypothesis=="E"){
        #Loop over exposed and unexposed individuals. 
        for(comb in 1:2){
            if(comb==1){
                E<-1
                freqE<-fE
            }else if(comb==2){
                E<-0
                freqE<-1-fE
            }   
            probD<-probD+exp(alphaP0+beta2*E)/(1+exp(alphaP0+beta2*E))*freqE
        }
    }
    return(probD)
}
