#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Maximization of the expected log-likelihoods L0 and L1 under the null-hypothesis and the alternative and calculation of lambda. 
ML_marginalE<-function(fE,alphaP1_marginalE,beta,no_LD){
    
    #aa is the likelihood function for the case. 
    aa<-rep(NA,no_LD)
    #bb is the likelihood function for the control. 
    bb<-rep(NA,no_LD)
    #cc is the penetrance function based on the logistic model, i.e. the probability of beeing diseased given the genotype combination of each matched pair. 
    cc<-rep(NA,no_LD)
    #multiplier is one of 9 (=3*3=number of genotype combinations of all case-control pairs) summands which together with the conditional likelihood form the nominator of the expected log-likelihood. 
    multiplier<-matrix(NA,no_LD,4)
    #summand is the sum of the products of the penetrance function, and the genotype frequencies for all 9 combinations of matched pairs and forms the denominator of the expected log-likelihood. 
    summand<-rep(0,no_LD)
    #Maximum likelihood value under H1. 
    max_lik_H1<-rep(NA,no_LD)
    #Maximum likelihood value under H0. 
    max_lik_H0<-rep(NA,no_LD)

    
    
     #Loop over all LD-values. 
    for(loop in 1:no_LD){
        count<-0
       #Loop over all case-control combinations (2*2=4). 
        for(case_comb in 1:2){
        
            #Assignment of the code for the environmental exposure to the case. 
            #Assignment of the frequency of the environmental exposure. 
            if(case_comb==1){
                Ef<-1
                freqEf<-fE
            }else if(case_comb==2){
                Ef<-0
                freqEf<-1-fE
            }
            
                     
            for(ctrl_comb in 1:2){
                count<-count+1
                #Assignation of code for the environmental exposure to the control. 
                #Assignment of the frequency of the environmental exposure. 
                if(ctrl_comb==1){
                    Ek<-1
                    freqEk<-fE
                }else if(ctrl_comb==2){
                    Ek<-0
                    freqEk<-1-fE
                }
                
                #aa is the "likelihood function" for the case. 
                aa[loop]<-exp(alphaP1_marginalE+beta[loop]*Ef)/(1+exp(alphaP1_marginalE+beta[loop]*Ef))
                #bb is the "likelihood function" for the control. 
                bb[loop]<-1-(exp(alphaP1_marginalE+beta[loop]*Ek)/(1+exp(alphaP1_marginalE+beta[loop]*Ek)))
                #cc is the penetrance function based on the logistic model, i.e. the probability of beeing diseased given the exposure combination of each matched pair. 
                cc[loop]<-aa[loop]*bb[loop]
                   
                #multiplier is one of 4 summands which together with the conditional likelihood form the nominator of the expected log-likelihood. 
                 multiplier[loop,count]<-cc[loop]*freqEf*freqEk
                #summand is the sum of the products of the penetrance function and the exposure frequencies for all 4 combinations of matched pairs and forms the denominator of the expected log-likelihood. 
                summand[loop]<-summand[loop]+(cc[loop]*freqEf*freqEk)
                
            }#End of ctrl_comb-for-loop. 
        }#End of case_comb-for-loop. 
        
        
        #L is the expected log-likelihood. 
        L<-function(b=beta[loop]){
            -1*((log(exp(b)/(exp(b)+exp(b)))*multiplier[loop,1]
            +log(exp(b)/(exp(b)+exp(0)))*multiplier[loop,2]
            +log(exp(0)/(exp(0)+exp(b)))*multiplier[loop,3]
            +log(exp(0)/(exp(0)+exp(0)))*multiplier[loop,4])/summand[loop])
        }
            
        #Parameter estimation by maximum likelihood method. 
        fit<-mle(L,start=list(b=beta[loop]),method="CG")
        #Maximum likelihood value under H1. 
        max_lik_H1[loop]<-logLik(fit)
        
        #Parameter estimation by maximum likelihood method. 
        fit0<-mle(L,fixed=list(b=0),method="CG")
        #Maximum likelihood value under H1. 
        max_lik_H0[loop]<-logLik(fit0)
    }    
    
    lambda<-rep(NA,no_LD)
    lambda<-2*(max_lik_H1-max_lik_H0)
    
    return(lambda)
}
