#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of the baseline risk of disease in unexposed individuals who do not carry the marker allele M using a Newton-type algorithm (nlm()). 
#The function to be minimized (fct()) originates from equation (3) in Gauderman, Statistics in Medicine, 2002, 21:35-50. 
P1_int<-function(or_eM, or_Em, or_EM, fM, fm, fE, fe, P0, probD, no_LD,mode_inh,r_path){
    probD_me<-rep(NA,no_LD)
    for(ii in 1:no_LD){
        alphaP0<-log(P0/(1-P0))
        beta1<-log(or_eM[ii])
        beta2<-log(or_Em[ii])
        int_ME<-or_EM[ii]/(or_eM[ii]*or_Em[ii])
        beta3<-log(int_ME)
        fM<-fM[1]
        fm<-fm[1]
        fE<-fE[1]
        fe<-fe[1]
        
        if(mode_inh=="dominant"){
            fct<-function(alphaP1,beta1,beta2,beta3,fM,fm,fE,fe,probD) abs(((exp(alphaP1+beta1+beta2+beta3)/(1+exp(alphaP1+beta1+beta2+beta3))*fM^2*fE+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*fM^2*fe+exp(alphaP1+beta1+beta2+beta3)/(1+exp(alphaP1+beta1+beta2+beta3))*2*fM*fm*fE+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*2*fM*fm*fe+exp(alphaP1+beta2)/(1+exp(alphaP1+beta2))*fm^2*fE+exp(alphaP1)/(1+exp(alphaP1))*fm^2*fe)-probD))
        }else if(mode_inh=="recessive"){
            fct<-function(alphaP1,beta1,beta2,beta3,fM,fm,fE,fe,probD) abs(((exp(alphaP1+beta1+beta2+beta3)/(1+exp(alphaP1+beta1+beta2+beta3))*fM^2*fE+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*fM^2*fe+exp(alphaP1+beta2)/(1+exp(alphaP1+beta2))*2*fM*fm*fE+exp(alphaP1)/(1+exp(alphaP1))*2*fM*fm*fe+exp(alphaP1+beta2)/(1+exp(alphaP1+beta2))*fm^2*fE+exp(alphaP1)/(1+exp(alphaP1))*fm^2*fe)-probD))        
        }else if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
            fct<-function(alphaP1,beta1,beta2,beta3,fM,fm,fE,fe,probD) abs(((exp(alphaP1+beta1*2+beta2+beta3*2)/(1+exp(alphaP1+beta1*2+beta2+beta3*2))*fM^2*fE+exp(alphaP1+beta1*2)/(1+exp(alphaP1+beta1*2))*fM^2*fe+exp(alphaP1+beta1+beta2+beta3)/(1+exp(alphaP1+beta1+beta2+beta3))*2*fM*fm*fE+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*2*fM*fm*fe+exp(alphaP1+beta2)/(1+exp(alphaP1+beta2))*fm^2*fE+exp(alphaP1)/(1+exp(alphaP1))*fm^2*fe)-probD))
        }
        loglik<-nlm(fct, p=alphaP0,beta1=beta1,beta2=beta2,beta3=beta3,fM=fM,fm=fm,fE=fE,fe=fe,probD=probD)
        alpha_start<-loglik$estimate
        probD_me[ii]<-exp(alpha_start)/(1+exp(alpha_start))
    }
    return(probD_me)
}
