#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of the baseline risk of disease in individuals who do not carry the marker alleleusing a Newton-type algorithm (nlm()). 
#The function to be minimized (fct()) originates from equation (3) in Gauderman, Statistics in Medicine, 2002, 21:35-50. 
P1_mainG<-function(or_M,fM,no_LD,P0,Kp,mode_inh){
    probD_m<-rep(NA,no_LD)
    fm<-1-fM
    if(length(P0)==1){
        for(ii in 1:no_LD){
            alphaP0<-log(P0/(1-P0))
            beta1<-log(or_M[ii])
            fM<-fM[1]
            fm<-fm[1]
            
            if(mode_inh=="dominant"){
                fct<-function(alphaP1,beta1,fM,fm,Kp) abs(((exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*fM^2+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*2*fM*fm+exp(alphaP1)/(1+exp(alphaP1))*fm^2)-Kp))
            }else if(mode_inh=="recessive"){
                fct<-function(alphaP1,beta1,fM,fm,Kp) abs(((exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*fM^2+exp(alphaP1)/(1+exp(alphaP1))*2*fM*fm+exp(alphaP1)/(1+exp(alphaP1))*fm^2)-Kp))
            }else if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                fct<-function(alphaP1,beta1,fM,fm,Kp) abs(((exp(alphaP1+2*beta1)/(1+exp(alphaP1+2*beta1))*fM^2+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*2*fM*fm+exp(alphaP1)/(1+exp(alphaP1))*fm^2)-Kp))
            }
            loglik<-nlm(fct, p=alphaP0,beta1=beta1,fM=fM,fm=fm,Kp=Kp)
            loglik
            alpha_start<-loglik$estimate
            probD_m[ii]<-exp(alpha_start)/(1+exp(alpha_start))
        } 
    }else{
        for(ii in 1:no_LD){
            alphaP0<-log(P0/(1-P0))
            beta1<-log(or_M[ii])
            fM<-fM[1]
            fm<-fm[1]
            
            if(mode_inh=="dominant"){
                fct<-function(alphaP1,beta1,fM,fm,Kp) abs(((exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*fM^2+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*2*fM*fm+exp(alphaP1)/(1+exp(alphaP1))*fm^2)-Kp))
            }else if(mode_inh=="recessive"){
                fct<-function(alphaP1,beta1,fM,fm,Kp) abs(((exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*fM^2+exp(alphaP1)/(1+exp(alphaP1))*2*fM*fm+exp(alphaP1)/(1+exp(alphaP1))*fm^2)-Kp))
            }else if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                fct<-function(alphaP1,beta1,fM,fm,Kp) abs(((exp(alphaP1+2*beta1)/(1+exp(alphaP1+2*beta1))*fM^2+exp(alphaP1+beta1)/(1+exp(alphaP1+beta1))*2*fM*fm+exp(alphaP1)/(1+exp(alphaP1))*fm^2)-Kp))
            }
            loglik<-nlm(fct, p=alphaP0[ii],beta1=beta1,fM=fM,fm=fm,Kp=Kp)
            loglik
            alpha_start<-loglik$estimate
            probD_m[ii]<-exp(alpha_start)/(1+exp(alpha_start))
        } 
    }
}
