#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

indirG<-function(fT,or_T,fM_vec,LD_measure,LD_set,P0,Kp,alpha,test,power,N,mode_inh,r_path,data_path,outputfile,par_spec){
    
    mainG_path<-paste(r_path,"mainG.R",sep="")
    source(mainG_path)
        
    #Function parameters are checked for correctness. 
    if(fT<=0|fT>=1){
        stop("fT out of range:\t",fT,"\n")
    }
    if(or_T<=0){
        stop("or_T out of range:\t",or_T,"\n")
    }
    if(any(fM_vec<=0|fM_vec>=1)){
        stop("At least one value of fM is out of range:\t",fM_vec,"\n")
    }
    if(LD_measure!="D"&LD_measure!="r2"){
        stop("Invalid LD measure:\t",LD_measure,"\n")
    }
    if(LD_measure=="r2"){
        if(any(LD_set<0)|any(LD_set>(1+10^(-12)))){
            stop("r2 values should be in the range 0 to 1!")
        }
    }
    if(LD_measure=="D"){
        if(any(LD_set>(1+10^(-12))|LD_set<(-1-10^(-12)))){
            stop("D values should be in the range -1 to 1!")
        }
    }
     if(alpha>0.5|alpha<10^(-12)){
        stop("Significance level alpha should be in the range (10^(-12),0.5). Your specification: \t",alpha,"\n")
    }
    if(test!=1&test!=2){
        stop("Wrong test value:\t",test,"\n")
    }
    if(power==FALSE&N==FALSE){
        stop("Either desired power or the available sample size needs to be specified!")
    }
    if(power!=FALSE&N!=FALSE){
        stop("Specify EITHER power OR sample size!")
    }
    if(power!=FALSE&power>0.999999){
        stop("Power should be in the range (0.01,0.999999)! Your specification: \t",power,"\n")
    }
    if(power!=FALSE&power<0.01){
        stop("Power should be in the range (0.01,0.999999)! Your specification: \t",power,"\n")
    }
    if(N!=FALSE&N<1){
        stop("The available sample size should be at least 1! Your specification: \t",N,"\n")
    }
    if(mode_inh!="dominant"&mode_inh!="recessive"&mode_inh!="multiplicative"&mode_inh!="log-additive"){
        stop("Invalid mode of inheritance:\t",mode_inh,"\n")
    }
    #Tests if the number of marker allele frequencies and the number of LD parameters fit. 
    no_fM<-length(fM_vec)
    if(is.matrix(LD_set)){
        no_fM_in_LD<-dim(LD_set)[1]
        no_LD<-dim(LD_set)[2]
        if(no_fM!=no_fM_in_LD){
            stop("Discordance between number of markers and number of corresponding LD values!\n","fM_vec:\t",fM_vec,"\n LD_set:\t",LD_set)
        }
    }else if(no_fM>1){
        LD_length<-length(LD_set)
        if(LD_length!=no_fM){
        stop("Discordance between number of markers and number of corresponding LD values!\n","fM_vec:\t",fM_vec,"\n","\n LD_set:\t",LD_set)
        }
        no_LD<-1
    }else if(no_fM==1){
        no_LD<-length(LD_set)
    }
    
    if(N==FALSE){
        result_name<-"sample size"
    }else{
        result_name<-"power"
    }
    
    no_row<-no_LD*no_fM
    Output<-matrix(NA,no_row,4)
    colnames(Output)<-c("fM","d_TM","r2",result_name)
    cc<-1
    #Loop over all marker allele frequencies. 
    for(fM_index in 1:no_fM){
        fM<-fM_vec[fM_index]
        if(is.matrix(LD_set)){
            LD<-LD_set[fM_index,]
        }else if(no_fM>1){
            LD<-LD_set[fM_index]
        }else if(no_fM==1){
            LD<-LD_set
        }
        #Estimation of OR's as well as power and sample size; results will be stored in files. 
        Output[c(cc:(no_LD+cc-1)),]<-mainG(fT,or_T,fM,no_LD,LD,LD_measure,P0,Kp,alpha,test,power,N,mode_inh,r_path,data_path,outputfile,par_spec)
        cc<-cc+no_LD
    }
    
    return(Output)
}
