#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

mainE<-function(fE,or_E,P0,Kp,alpha,test,power,N,r_path,data_path,outputfile,par_spec){
    
    Power_file<-paste(r_path,"Power_mainE.R",sep="")
    source(Power_file)
    
    #Function parameters are checked for correctness. 
    if(fE<=0|fE>=1){
        stop("fE out of range:\t",fE,"\n")
    }
    if(or_E<=0){
        stop("or_E out of range:\t",or_E,"\n")
    }
    if(alpha>0.5|alpha<10^(-12)){
        stop("Significance level alpha should be in the range (10^(-12),0.5). Your specification: \t",alpha,"\n")
    }
    if(test!=1&test!=2){
        stop("Wrong test value:\t",test,"\n")
    }
    if(power==FALSE&N==FALSE){
        stop("Either desired power or the available sample size needs to be specified!")
    }
    if(power!=FALSE&N!=FALSE){
        stop("Specify EITHER power OR sample size!")
    }
    if(power!=FALSE&power>0.999999){
        stop("Power should be in the range (0.01,0.999999)! Your specification: \t",power,"\n")
    }
    if(power!=FALSE&power<0.01){
        stop("Power should be in the range (0.01,0.999999)! Your specification: \t",power,"\n")
    }
    if(N!=FALSE&N<1){
        stop("The available sample size should be at least 1! Your specification: \t",N,"\n")
    }
    if(power==FALSE){
        result_name<-"Power"
        result_name_out<-"power"
    }else{
        result_name<-"SampleSize"
        result_name_out<-"sample size"
    }

    
    #Estimation of power and sample size; results will be stored in files. 
    alphaP0<-log(P0/(1-P0))
    result0<-Power_mainE(fE,or_E,alphaP0,alpha,test,power,N,r_path)
    P0.x<-round(P0,digits=5)
    Kp.x<-round(Kp,digits=5)
    result<-cbind(fE,or_E,result0,P0.x,Kp.x)
    P0_round<-round(P0,digits=2)
    Kp_round<-round(Kp,digits=2)
    if(par_spec=="P0"){
        result_file<-paste(outputfile,result_name,"_mainE_fE_",fE,"_or_E_",or_E,"_P0_",P0_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,".out",sep="")
    }else{
        result_file<-paste(outputfile,result_name,"_mainE_fE_",fE,"_or_E_",or_E,"Kp",Kp_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,".out",sep="")
    }
    write.table(result,result_file,row.names=FALSE,col.names=c("fE","or_E",result_name,"P0","Kp"))
    
    out<-paste(result_name_out, ": ", result0, sep="")
    return(out)                 
}   
