#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

mainG<-function(fT,or_T,fM,no_LD,LD,LD_measure,P0,Kp,alpha,test,power,N,mode_inh,r_path,data_path,outputfile,par_spec){
    
    #OR estimation. 
    ORM_path<-paste(r_path,"ORM.R",sep="")
    source(ORM_path)
    
    #Power/sample size estimation. 
    Power_file<-paste(r_path,"Power_mainG.R",sep="")
    source(Power_file)
            
    
    #OR estimation. 
    Output<-ORM(fT,or_T,fM,no_LD,LD,LD_measure,P0,Kp,mode_inh,r_path)
    
    #Result file for OR's. 
    P0.x<-round(P0,digits=5)
    Kp.x<-round(Kp,digits=5)
    P0.xx<-rep(P0.x,no_LD)
    Kp.xx<-rep(Kp.x,no_LD)
    Output_round<-round(Output,digits=5)
    result_or<-cbind(Output_round,P0.xx,Kp.xx)
    result_or_out<-matrix(result_or,no_LD,9)

    P0_round<-round(P0,digits=2)    
    Kp_round<-round(Kp,digits=2)    
    if(par_spec=="P0"){
        file_name<-paste(data_path,"MainG_fM_",fM,"_fT_",fT,"_or_T_",or_T,"_P0_",P0_round,".out",sep="")
    }else{
        file_name<-paste(data_path,"MainG_fM_",fM,"_fT_",fT,"_or_T_",or_T,"_Kp_",Kp_round,".out",sep="")
    }
    write.table(result_or_out, file=file_name,row.names=FALSE,col.names=c("fT","fM","D","r2","or_T","or_M","P1","P0","Kp")) 
    
       
    #Sample size/power estimation. 
    or_M<-Output[,6]
    alphaP1<-log(Output[,7]/(1-Output[,7]))
    d_TM<-Output[,3]
    d_TM_round<-round(d_TM,digits=5)
    r2<-Output[,4]
    r2_round<-round(r2,digits=5)
    result0<-Power_mainG(fM,or_M,alphaP1,alpha,test,power,N,mode_inh,r_path,no_LD)    
    result<-cbind(fM,d_TM_round,r2_round,result0)
    
    #Result file for power/sample size. 
    if(power==FALSE){result_name<-"Power"
    }else{result_name<-"SampleSize"}  
    if(par_spec=="P0"){ 
        result_file<-paste(outputfile,result_name,"_mainG_fM_",fM,"_fT_",fT,"_or_T_",or_T,"_P0_",P0_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,"_mode_inh_",mode_inh,".out",sep="")
    }else{
        result_file<-paste(outputfile,result_name,"_mainG_fM_",fM,"_fT_",fT,"_or_T_",or_T,"_Kp_",Kp_round,"_alpha_",alpha,"_test_",test,"_power_",power,"_N_",N,"_mode_inh_",mode_inh,".out",sep="")
    } 
    write.table(result,result_file,row.names=FALSE,col.names=c("fM","D","r2",result_name))
    
    return(result)          
}
