#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of Pr(D=1|E=1)*Pr(D=0|E=0)/(Pr(D=0|E=1)*Pr(D=1|E=0))
marginalE<-function(or_M,or_E,int_ME,fM,no_LD,alphaP1,mode_inh){

    beta<-matrix(NA,3,no_LD)
    beta[1,]<-log(or_M)
    beta[2,]<-log(or_E)
    beta[3,]<-log(int_ME)
    
    prob_DE<-rep(0,no_LD)
    prob_dE<-rep(0,no_LD)
    prob_De<-rep(0,no_LD)
    prob_de<-rep(0,no_LD)
    
    for(comb in 1:3){
        #Loop over all genotypes (3 possibilities). 
        if(comb==1){
            g<-"AA"
            freqT<-fM^2
            if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                G<-2
            }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                G<-1
            }
        }else if(comb==2){
            g<-"aA"
            freqT<-2*fM*(1-fM)
            if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                G<-1
            }else if(mode_inh=="recessive"){
                G<-0
            }
        }else if(comb==3){
            g<-"aa"
            freqT<-(1-fM)^2
            G<-0
        }
        prob_DE<-prob_DE+(exp(alphaP1+beta[1,]*G+beta[2,]+beta[3,]*G)/(1+exp(alphaP1+beta[1,]*G+beta[2,]+beta[3,]*G))*freqT)
        prob_dE<-prob_dE+((1-(exp(alphaP1+beta[1,]*G+beta[2,]+beta[3,]*G)/(1+exp(alphaP1+beta[1,]*G+beta[2,]+beta[3,]*G))))*freqT)
        prob_De<-prob_De+(exp(alphaP1+beta[1,]*G)/(1+exp(alphaP1+beta[1,]*G))*freqT)
        prob_de<-prob_de+((1-(exp(alphaP1+beta[1,]*G)/(1+exp(alphaP1+beta[1,]*G))))*freqT)
    }
    OR_E_bar<-prob_DE*prob_de/(prob_dE*prob_De)
    
    return(OR_E_bar)
}
